<?php
/**
 * NLSClientScript class v1.0
 *
 * @author nlac <nlacsoft@gmail.com>
 * @link http://nlacsoft.net
 * @license http://nlacsoft.net
 */

/**
 * CClientScript extension for handling managed lazy loading of javascript files
 */
class NLSClientScript extends CClientScript
{

	private static function scriptFile($f) {
		return CHtml::script('if (typeof jQuery != "undefined" && jQuery.yii && jQuery.yii.lazyLoad) jQuery.yii.lazyLoad("' . $f . '");');
	}
	
	private static function cssFile($f, $media) {
		return CHtml::script('if (typeof jQuery != "undefined" && jQuery.yii && jQuery.yii.lazyLoad) jQuery.yii.lazyLoad("' . $f . '", {media:"' . $media . '"});');
	}
	
/**
 * Generates usual linking code of base scripts
 * @return the html linking code
 */
	protected static function _renderBaseScriptFiles() {
		if (!Yii::app()->request->isAjaxRequest) {

			$cs = Yii::app()->getClientScript();
			
			//Deploying system assets statically
			$assetsPath = Yii::app()->getAssetManager()->publish( Yii::getPathOfAlias( 'system.web.js'), false, -1, @Yii::app()->params['dev'] );
			
			$scripts = array(
				$assetsPath . '/source/jquery.js',
				$assetsPath . '/source/jquery.yii.js'
				//$assetsPath . '/ext/yii_ext.js'
			);
			
			$html = "";
			$scriptList = "[";
			foreach($scripts as $s) {
				$html .= CHtml::scriptFile($s) . "\n";
				$scriptList .= "\n\"$s\",";
			}
			$scriptList = rtrim($scriptList, ',') . ']';

			$html .= CHtml::script('
(function($){

$.extend($.yii, {
	_rescache : {},

	is_a : function(obj, _class) {
		return (typeof obj == "object" && obj && obj.constructor == _class);
	},
	
	removeAsset : function(f) {
		f.match(/[^\\/\\\\]+\.(js|css)$/);
		return RegExp.lastMatch;
	},
	
	fromCache : function(f) {
		f = this.removeAsset(f);
		//console.log(f);
		return this._rescache[ f ];
	},
	
	toCache : function(f) {
		if (this.is_a(f,Array)) {
			for(var i=0; i<f.length; i++)
				this.toCache(f[i]);
			return;
		}
		
		f = this.removeAsset(f);
		this._rescache[f] = f;
	},
	
	lazyLoad : function(f, opt) {
		if (this.is_a(f,Array)) {
			for(var i=0; i<f.length; i++)
				this.lazyLoad(f[i]);
			return;
		}
		
		if (this.fromCache(f))
			return;
		
		this.toCache(f);
		
		if (!opt)opt={};

		if (f.match(/\.js$/)) {
			//linking js
			$.ajax({
				async : false,
				type: "GET",
				url: f,
				dataType: "script"
			});
		}
		else
		if (f.match(/\.css$/)) {
			//linking css
			var lnk = document.createElement("link");		
			lnk.rel="stylesheet";
			lnk.type="text/css";
			lnk.media=opt.media||"screen";
			lnk.href=f;
			(document.getElementsByTagName("head"))[0].appendChild(lnk);
		}
	}

});

$.yii.toCache(' . $scriptList . ');

})(jQuery);
');

			return $html;
		}

		return '';
	}
	
	/**
	 * Inserts the scripts in the head section.
	 * @param string $output the output to be inserted with scripts.
	 */
	public function renderHead(&$output)
	{
		$html='';
		
		//nlac:!!!
		$html .= self::_renderBaseScriptFiles();
		
		foreach($this->metaTags as $meta)
			$html.=CHtml::metaTag($meta['content'],null,null,$meta)."\n";
		foreach($this->linkTags as $link)
			$html.=CHtml::linkTag(null,null,null,null,$link)."\n";
//		foreach($this->cssFiles as $url=>$media)
//			$html.=CHtml::cssFile($url,$media)."\n";
//		foreach($this->css as $css)
//			$html.=CHtml::css($css[0],$css[1])."\n";
		if($this->enableJavaScript)
		{
			if(isset($this->scriptFiles[self::POS_HEAD]))
			{
				foreach($this->scriptFiles[self::POS_HEAD] as $scriptFile)
					//$html.=CHtml::scriptFile($scriptFile)."\n";
					$html.=self::scriptFile($scriptFile)."\n";
			}

			if(isset($this->scripts[self::POS_HEAD]))
				$html.=CHtml::script(implode("\n",$this->scripts[self::POS_HEAD]))."\n";
		}
		
		//nlac: css moved to here
		foreach($this->cssFiles as $url=>$media)
			$html.=self::cssFile($url,$media)."\n";
		foreach($this->css as $css)
			$html.=CHtml::css($css[0],$css[1])."\n";

		if($html!=='')
		{
			$count=0;
			$output=preg_replace('/(<title\b[^>]*>|<\\/head\s*>)/is','<###head###>$1',$output,1,$count);
			if($count)
				$output=str_replace('<###head###>',$html,$output);
			else
				$output=$html.$output;
		}
	}

	/**
	 * Inserts the scripts at the beginning of the body section.
	 * @param string $output the output to be inserted with scripts.
	 */
	public function renderBodyBegin(&$output)
	{
		$html='';
		if(isset($this->scriptFiles[self::POS_BEGIN]))
		{
			foreach($this->scriptFiles[self::POS_BEGIN] as $scriptFile)
			//$html.=CHtml::scriptFile($scriptFile)."\n";
			$html.=self::scriptFile($scriptFile)."\n";
		}
		if(isset($this->scripts[self::POS_BEGIN]))
			$html.=CHtml::script(implode("\n",$this->scripts[self::POS_BEGIN]))."\n";

		if($html!=='')
		{
			$count=0;
			$output=preg_replace('/(<body\b[^>]*>)/is','$1<###begin###>',$output,1,$count);
			if($count)
				$output=str_replace('<###begin###>',$html,$output);
			else
				$output=$html.$output;
		}
	}

	/**
	 * Inserts the scripts at the end of the body section.
	 * @param string $output the output to be inserted with scripts.
	 */
	public function renderBodyEnd(&$output)
	{
		if(!isset($this->scriptFiles[self::POS_END]) && !isset($this->scripts[self::POS_END])
			&& !isset($this->scripts[self::POS_READY]) && !isset($this->scripts[self::POS_LOAD]))
			return;

		$fullPage=0;
		$output=preg_replace('/(<\\/body\s*>)/is','<###end###>$1',$output,1,$fullPage);
		$html='';
		if(isset($this->scriptFiles[self::POS_END]))
		{
			foreach($this->scriptFiles[self::POS_END] as $scriptFile)
				//$html.=CHtml::scriptFile($scriptFile)."\n";
				$html.=self::scriptFile($scriptFile)."\n";
		}
		$scripts=isset($this->scripts[self::POS_END]) ? $this->scripts[self::POS_END] : array();
		if(isset($this->scripts[self::POS_READY]))
		{
			if($fullPage)
				$scripts[]="jQuery(function($) {\n".implode("\n",$this->scripts[self::POS_READY])."\n});";
			else
				$scripts[]=implode("\n",$this->scripts[self::POS_READY]);
		}
		if(isset($this->scripts[self::POS_LOAD]))
		{
			if($fullPage)
				$scripts[]="jQuery(window).load(function() {\n".implode("\n",$this->scripts[self::POS_LOAD])."\n});";
			else
				$scripts[]=implode("\n",$this->scripts[self::POS_LOAD]);
		}
		if(!empty($scripts))
			$html.=CHtml::script(implode("\n",$scripts))."\n";

		if($fullPage)
			$output=str_replace('<###end###>',$html,$output);
		else
			$output=$output.$html;
	}

}
